@@ -5,8 +5,6 @@ use warnings;
 
 use parent 'PLS::Server::Request';
 
-use Coro;
-
 use PLS::Parser::Document;
 use PLS::Server::Request::Diagnostics::PublishDiagnostics;
 
@@ -25,7 +23,7 @@ sub service
 {
     my ($self, $server) = @_;
 
-    $server->{server_requests}->put(PLS::Server::Request::Diagnostics::PublishDiagnostics->new(uri => $self->{params}{textDocument}{uri}));
+    $server->send_server_request(PLS::Server::Request::Diagnostics::PublishDiagnostics->new(uri => $self->{params}{textDocument}{uri}));
 
     return;
 } ## end sub service
